#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# rtorstat.py - A simple rtorrent status web page generator.
#
# Copyright (c) George Notaras <gnot [AT] g-loaded [DOT] eu>
#
# Project home: http://www.g-loaded.eu/2007/06/23/rtorstat-a-simple-rtorrent-status-web-page-generator/
#
# License: GPLv2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the:
#
# Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston,
# MA 02111-1307  USA
#

__version__ = "0.3"

import sys, os.path, glob, datetime
from string import Template

# Requirement
try:
	from bencode import bdecode
except ImportError:
	try:
		from BitTorrent.bencode import bdecode
	except ImportError:
		sys.stderr.write("%s: ERROR: Missing dependency\nbencode.py was not found in current directory or in local BitTorrent Client (Mainline) installation.\n" % sys.argv[0])
		sys.stderr.flush()
		sys.exit(1)


status_page = Template("""<html>
<head>
	<META NAME="GOOGLEBOT" CONTENT="NOARCHIVE">
	<META NAME="ROBOTS" CONTENT="NOINDEX,NOFOLLOW">
	<META HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">
	<META HTTP-EQUIV="CACHE-CONTROL" CONTENT="NO-CACHE">
	<META HTTP-EQUIV="EXPIRES" CONTENT="Mon, 22 Jul 2000 01:01:01 GMT">
	<title>rTorrent Status</title>
</head>
<body>
	<div style="margin:20px;">
		<h2>rTorrent Status</h2>
		<pre>${pagehead}</pre>
		<br />
		<strong>Started/Stopped Torrents</strong>
		<pre>${torstarted}</pre>
		<br />
		<strong>Finished Torrents</strong>
		<pre>${torfinished}</pre>
	</div>

	<br /><br />
	<div style="width:200px; margin:20px;">
		<hr />
		<em><span style="font-size:80%;">Created with rtorstat v${version}</span></em>
	</div>

</body>
<head>
	<META NAME="GOOGLEBOT" CONTENT="NOARCHIVE">
	<META NAME="ROBOTS" CONTENT="NOINDEX,NOFOLLOW">
	<META HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">
	<META HTTP-EQUIV="CACHE-CONTROL" CONTENT="NO-CACHE">
	<META HTTP-EQUIV="EXPIRES" CONTENT="Mon, 22 Jul 2000 01:01:01 GMT">
</head>
</html>
""")


def sz(size):
	if size < 1024**2:
		return "%.2f Kb" % (float(size)/float(1024))
	else:
		if size < 1024**3:
			return "%.2f MB" % (float(size)/float(1024**2))
		else:
			return "%.2f GB" % (float(size)/float(1024**3))

def started_stopped(state, complete):
	if complete:
		return "FINISHED"
	if state:
		return "STARTED"
	return "STOPPED"

def ignores(ignores_orders):
	if ignores_orders:
		return "(IGNORES ORDERS)"
	return ""

def get_status(torrent):
	file_output = []
	f = open(torrent)
	t_dict = bdecode(f.read())
	f.close()
	#print t_dict['rtorrent']
	#del t_dict['info']['pieces']
	#print t_dict
	t_name = t_dict['info']['name']
	try:
		t_size = sum(int(tfile['length']) for tfile in t_dict['info']['files'])
	except:
		t_size = t_dict['info']['length']	# when there is only one file in the torrent
	t_chunk_len = t_dict['info']['piece length']
	total_downloaded = t_dict['rtorrent']['chunks_done'] * t_chunk_len
	total_uploaded = t_dict['rtorrent']['total_uploaded']
	state_changed = int(t_dict['rtorrent']['state_changed'])
	t_state = t_dict['rtorrent']['state']
	t_complete = t_dict['rtorrent']['complete']
	t_ignores_orders = t_dict['rtorrent']['ignore_commands']
    
	share_ratio = 0
	complete_percent = 0

	if total_uploaded > 0 and total_downloaded > 0:
		share_ratio = float(total_uploaded)/float(total_downloaded)

	if not t_complete and total_downloaded > 0:
		complete_percent = (float(total_downloaded)/float(t_size))*100

	if t_complete:
		complete_percent = 100
		total_downloaded = t_size

	#file_output.append("-"*100)
	file_output.append("%s - %s %s:" % (t_name, started_stopped(t_state, t_complete), ignores(t_ignores_orders)))
	file_output.append( "size: %s - downloaded: %s - uploaded: %s - complete: %.2f%% - ratio: %.2f\n" % \
		( sz(t_size), sz(total_downloaded), sz(total_uploaded), complete_percent, share_ratio ) )

	if t_complete:
		ret_status = "FINISHED"
	else:
		ret_status = None

	return ret_status, "\n".join(file_output)

def add_num_to_list_items(mylist):
	for k in range(len(mylist)):
		mylist[k] = "%d. - %s" % (k+1, mylist[k])
	return mylist

def usage():
	sys.stderr.write("Usage:\n%s /path/to/session/dir/\n\n" % os.path.basename(sys.argv[0]))
	sys.stderr.flush()
	sys.exit(1)

def main():

	if "-h" in sys.argv or "--help" in sys.argv:
		usage()
	elif len(sys.argv) == 2:
		session_dir = sys.argv[1]
		if not os.path.exists(session_dir) or not os.path.isdir(session_dir):
			sys.stderr.write("ERROR: session path does not exist or is not a directory.\n\n")
			sys.stderr.flush()
			usage()
	else:
		usage()

	pagehead = []
	started_list = []
	finished_list = []

	pagehead.append("Last Update: %s\n" % datetime.datetime.now().strftime("%a, %b %d %Y, %H:%M:%S"))

	session_list = glob.glob(os.path.join(session_dir, "*.torrent"))

	for torrent in session_list:
		tor_status, tor_text = get_status(torrent)
		if tor_status == "FINISHED":
			finished_list.append(tor_text)
		else:
			started_list.append(tor_text)

	pagehead = "\n".join(pagehead)
	if started_list:
		started_list = add_num_to_list_items(started_list)
		torstarted = "\n".join(started_list)
	else:
		torstarted = "N/A"
	if finished_list:
		finished_list = add_num_to_list_items(finished_list)
		torfinished = "\n".join(finished_list)
	else:
		torfinished = "N/A"

	sys.stdout.write(status_page.substitute(pagehead=pagehead,
						torstarted=torstarted,
						torfinished=torfinished,
						version=__version__))
	sys.stdout.flush()


if __name__ == '__main__':
	main()

